// -------------------------------
// PART ONE: define common tools, constants
// -------------------------------
// used when testing in Chrome, rather than in MobileSafari
DEBUG = (window.location.href.indexOf('debug') > -1);

Q = function(s){
    return encodeURIComponent(s);
}

B = function(path) {
    // this is the magical hack that lets javascript send strings to Objective-C
    if (DEBUG) {
//        alert(path);
        return;
    }
    window.location.href = 'bridge:///' + path;
}

var isAndroid = (navigator.userAgent.match(/Android/i));

REMOTE_LOG = function(s){
    $.ajax({
        url: '/logger/',
        type: 'POST',
        data: {'pgid': GAMESTATE.PGID, 's': s},
    });
};

// Getting Query Params
// http://stackoverflow.com/questions/901115/how-can-i-get-query-string-values-in-javascript
// Usage: $.QueryString["param"]
(function($) {
 $.QueryString = (function(a) {
                  if (a == "") return {};
                  var b = {};
                  for (var i = 0; i < a.length; ++i)
                  {
                  var p=a[i].split('=');
                  if (p.length != 2) continue;
                  b[p[0]] = decodeURIComponent(p[1].replace(/\+/g, " "));
                  }
                  return b;
                  })(window.location.search.substr(1).split('&'))
 })(jQuery);

READY_FOR_USER = false;
READY_TO_PLAY = false;
IS_HOURLY = $.QueryString["style"] === '4hour';
IS_EXPENSIVE = $.QueryString["cost"] === 'double';

// -------------------------------
// PART TWO: functions to data from Obj-C
// -------------------------------

// Local Notificactions
SCHEDULED = false;
schedule_local_notif = function(message, secondsFromNow){
    if (SCHEDULED) return;
    B('reminder?identifier=timegate&message=' + Q(message)
      + '&secondsFromNow=' + secondsFromNow);
    SCHEDULED = true;
};

did_reminder = function(){};

unschedule_local_notif = function(message, secondsFromNow){
    B('unschedulereminder?identifier=timegate');
};

did_unschedulereminder = function(){};

// Pulling data from Obj-C
do_render = function(){
    // "render" is a name from the client
    // similar to template rendering in Make or Django.
    // we use it here to get strings and counters from the user's gamestate

    // Make subquery for objectinfo
    var storyQuery = Q('{"identifier":"[StoryID]", "infoClass":"StoryInfo"}');

    B('render?'
    + 'FREE_EPISODES=' + Q('[FREE_EPISODES]')
    + '&PASSES=' + Q('[PASSES]')
    + '&SECONDS_TO_WAIT=' + Q('[TIME_LEFT:DAILY_BONUS]')
    + '&ChapterNum=' + Q('[ChapterNum]')
    + '&StoryID=' + Q('[StoryID]')
    + '&SupportID=' + Q('[SupportID]')
    + '&PGID=' + Q('[PGID]')
    + '&ReferralCode=' + Q('[ReferralCode]')
    + '&ReferredFriends=' + Q('[NumFriends]')
    + '&EarnedReferralRewards=' + Q('[EarnedReferralRewards]')
    + '&CurrencyUnit=' + Q('[CurrencyUnit]')
    + '&objectinfo=' + storyQuery
    );
};

did_render = function(results){
    GAMESTATE = {};
    GAMESTATE.FREE_EPISODES = parseInt(results.FREE_EPISODES) || 0;
    GAMESTATE.PASSES = parseInt(results.PASSES) || 0;
    GAMESTATE.ChapterNum = results.ChapterNum;
    GAMESTATE.StoryID = results.StoryID;
    GAMESTATE.SupportID = results.SupportID;
    GAMESTATE.PGID = results.PGID;
    GAMESTATE.ReferralCode = results.ReferralCode;
    GAMESTATE.ReferredFriends = results.ReferredFriends;
    GAMESTATE.EarnedReferralRewards = results.EarnedReferralRewards;
    GAMESTATE.CurrencyUnit = results.CurrencyUnit;
    GAMESTATE.StoryInfo = results.objectinfo;

    GAMESTATE._time_of_last_load = (new Date());

    if (IS_HOURLY === false) {
        GAMESTATE.SECONDS_TO_WAIT = parseInt(results.SECONDS_TO_WAIT) || 0;
        combine_app_and_server_info();
    } else {
        render_countdown();
    }
};

// Countdown
render_countdown = function(){
    // we have to know the name of this story, so this is a second step.
    B('render/start_countdown?'
    + 'SECONDS_TO_WAIT=' + Q('[TIME_LEFT:BLOCKED_ON_' + GAMESTATE.StoryID + ']')
    );

};

start_countdown = function(results){
    GAMESTATE.SECONDS_TO_WAIT = parseInt(results.SECONDS_TO_WAIT) || 0;
    combine_app_and_server_info();
};

// Callback from IAP (hardcoded in client; I think)
iap_success = function(iap_identifier){
    // If an iap completes, do_render to make sure we absorb any changes instead of overwriting them
    do_render();

    // use another callback, otherwise alert() will freeze the game
    setTimeout(function(){
        // back to the timegate (is timeout still needed?)
        BUTTONS.back();
    }, 1);

    return "";
};
// -------------------------------
// PART THREE: IAPs
// -------------------------------

/*
:: IAP definitions ::
iap_1_premiumchoice_1 => $0.99
iap_1_premiumchoice_2 => $1.99
iap_1_premiumchoice_3 => $2.99
iap_1_premiumchoice_4 => $3.99
iap_1_premiumchoice_5 => $4.99
iap_1_premiumchoice_6 => $5.99
iap_1_premiumchoice_7 => $6.99
iap_1_premiumchoice_8 => $7.99
iap_1_premiumchoice_9 => $8.99
iap_1_premiumchoice_10 => $9.99
*/

if (IS_EXPENSIVE) {
    IAPS = {
        "iap_1_premiumchoice_1": {"price":"$0.99", "passes": 3},
        "iap_1_premiumchoice_2": {"price":"$1.99", "passes": 7},
        "iap_1_premiumchoice_5": {"price":"$4.99", "passes": 18},
        "iap_1_premiumchoice_10": {"price":"$9.99", "passes": 37},
    }
} else {
    IAPS = {
        "iap_1_premiumchoice_1": {"price":"$0.99", "passes": 5},
        "iap_1_premiumchoice_2": {"price":"$1.99", "passes": 11},
        "iap_1_premiumchoice_5": {"price":"$4.99", "passes": 30},
        "iap_1_premiumchoice_10": {"price":"$9.99", "passes": 75},
    }
}

$('#passesForIAP1').html(IAPS.iap_1_premiumchoice_1.passes);
$('#passesForIAP2').html(IAPS.iap_1_premiumchoice_2.passes);
$('#passesForIAP5').html(IAPS.iap_1_premiumchoice_5.passes);
$('#passesForIAP10').html(IAPS.iap_1_premiumchoice_10.passes);

// JS now only forwards back to the Gate on success

// -------------------------------
// PART FOUR: Make it work on iOS
// -------------------------------


// Hack to scale the screen for iPads
// since we don't have a responsive web-design yet!
if (!isAndroid && window.screen.width > 320){
   var scale = window.screen.width / 320.0;
   var val = "width=320px, initial-scale=" + scale + ", user-scalable=0";
   document.getElementById('viewport').setAttribute("content", val);
}

// On load, attach handlers to buttons
// Uses the touches API to be more responsive
$(function(){

    $('.btn').bind('touchstart', function(){
        this.classList.add('hover');
    });
    $('.btn').bind('touchend', function(){
        this.classList.remove('hover');
        CLICK_ON_ELEMENT(this.id);
    });
    $('.btn').bind('click', function(){ CLICK_ON_ELEMENT(this.id); });

    // start the data load, but only if we are in ios-webview land
    if (DEBUG){
        LOG('faking it!');
        did_render({
            "PGID":"matt",
            "PASSES":"2",
            "SECONDS_TO_WAIT":"15",
            "StoryID":"love.life",
            "StoryName":"Hollywood Crush",
            "ChapterNum":"2",
        });
    } else {
        do_render();
    }

    // disable touches temporarily after loading a page
    // otherwise errant touch-ends seem to get fired on launch
    // when switching pages
    prevent_double_touch();
});


TOUCHES_DISABLED = true;
prevent_double_touch = function(){
    TOUCHES_DISABLED = true;
    setTimeout(function(){
        TOUCHES_DISABLED = false;
    }, 1000);
}


window.onerror = function(msg, url, linenumber) {
    setTimeout(function(){
        var error = 'Error message: '+msg+'\nURL: '+url+'\nLine Number: '+linenumber;
        alert(error);
        REMOTE_LOG(error);
    }, 1);
    return true;
}

// Prevent scrolling
/*
document.ontouchmove = function(event){
    event.preventDefault();
}
*/


// -------------------------------
// PART FIVE: functions specific to this page
// -------------------------------

_COUNT = 0;
LOG = function(s){
    _COUNT +=1;
    $('#log').html($('#log').html() + '<br>' + _COUNT + ' ' + s);
    console.log(s);
}

CLICK_ON_ELEMENT = function(id){
    if (TOUCHES_DISABLED){
        LOG('touches disabled');
        // i think the click handle fires on the next page...
        return
    }
    if (!READY_FOR_USER){
        LOG('not ready for user yet');
        return;
    }
    if (BUTTONS[id]) BUTTONS[id]();
    else alert('no button for ' + id);

    prevent_double_touch();
}


_SHOWN_MODAL = undefined;
SHOW_MODAL = function(id){
    _SHOWN_MODAL = id;
    $("#" + id).modal({
      "backdrop"  : "static",
      "keyboard"  : true,
      "show"      : true // ensure the modal is shown immediately
    });
}

HIDE_MODAL = function(id){
    if (!id) return;
    $('#' + id).modal('hide');
    _SHOWN_MODAL = undefined;
};

sync_state_then_callback = function(callback, extraQuery){
    if (SYNC_DISABLED === true) return;

    var js_state = Q('{"counters":{"PASSES":' + GAMESTATE.PASSES
                     + ', "FREE_EPISODES":' + GAMESTATE.FREE_EPISODES
                     + '}}');

    // todo: invent a new name each time?
    did_sync = callback;
    if (extraQuery) {
        B("set/did_sync?json=" + js_state + extraQuery);
    } else {
        B("set/did_sync?json=" + js_state);
    }
}

did_sync = function(){};

next = function(from){
    unschedule_local_notif();
    // have to wait...
    setTimeout(function(){
        // tell the native code to advance to the next node in the story
        LOG('starting episode!');
        B('next');
    }, 1);
}

combine_app_and_server_info = function(){
    if (!GAMESTATE) return;
    STORYDATA = GAMESTATE.StoryInfo;

    // Have default values if not in a story
    GAMESTATE.AuthorName = STORYDATA.authorName || 'Author';
    GAMESTATE.AuthorBio = STORYDATA.authorBiography || '';
    GAMESTATE.IconURL = 'http://pg-story.appspot.com/dcon/' + STORYDATA.iconImage;
    if (STORYDATA.iconImage === undefined) GAMESTATE.IconURL = '';
    GAMESTATE.StoryName = STORYDATA.displayName;
    GAMESTATE.EndOfContentNote = STORYDATA.endOfContentNote || 'Thanks for playing!';

    update_view();
}

update_view = function(){
    // update the UI to match the GAMESTATE
    $('.num_passes').html(GAMESTATE.PASSES);

    if (IS_HOURLY === false) {
        if (GAMESTATE.FREE_EPISODES > 0){
            $('#num_free_episodes').html(GAMESTATE.FREE_EPISODES);
            $('#free_episodes_per_day').html(FREE_EPISODES_PER_DAY);
            $('#free_episodes').show();
            $('#out_of_episodes').hide();
        } else {
            $('#out_of_episodes').show();
            $('#free_episodes').hide();
        }
    }
    // what is this?
    $('#counters').show();
    $('.ChapterNum').html(GAMESTATE.ChapterNum);
    $('.StoryName').html(GAMESTATE.StoryName);

    $('#AuthorBio').html(GAMESTATE.AuthorBio);
    $('#AuthorName').html(GAMESTATE.AuthorName);
    $('#EndOfContentNote').html(GAMESTATE.EndOfContentNote);

    $('#referral_code').html(GAMESTATE.ReferralCode);
    $('#friends_referred').html(GAMESTATE.ReferredFriends);
    $('#rewards_earned').html(GAMESTATE.EarnedReferralRewards);
    $('.currency_unit').html(GAMESTATE.CurrencyUnit);
    
    $('.icon').attr('src', GAMESTATE.IconURL);

    HIDE_MODAL(_SHOWN_MODAL);

    update_timers();

    LOG('showing!');
    READY_FOR_USER = true;
};

update_timers = function(){
    var dt = (new Date()) - GAMESTATE._time_of_last_load;
    var remaining_seconds = GAMESTATE.SECONDS_TO_WAIT - Math.floor(dt / 1000);

    $('#blocked').hide();
    $('#free_episodes').hide();
    $('#out_of_episodes').hide();
    
    if (IS_HOURLY) {
        if (remaining_seconds > 0){
            // continue countdown, show timer
            $('#blocked').show();
            $('.timer').html(format_timer(remaining_seconds));
            format_readable_timer(remaining_seconds);
            setTimeout(update_timers, 500);
            schedule_local_notif(GAMESTATE.StoryName + " is ready to play!", remaining_seconds);
        } else {
            // time is up, hide timer
            READY_TO_PLAY = true;
        }
    } else {
        if (GAMESTATE.FREE_EPISODES > 0) {
            // Show passes that can be used
            $('#free_episodes').show();
        } else if (remaining_seconds > 0){
            // continue countdown, show timer
            $('#out_of_episodes').show();
        }

        if (remaining_seconds <= 0){
            // If the countdown is complete... award passes
            if (GAMESTATE.FREE_EPISODES < FREE_EPISODES_PER_DAY) {
                // award currency and reset!
                GAMESTATE.FREE_EPISODES = FREE_EPISODES_PER_DAY;
                sync_state_then_callback(do_render);
            }
        } else {
            // Countdown carries on
            $('.timer').html(format_timer(remaining_seconds));
            format_readable_timer(remaining_seconds);
            setTimeout(update_timers, 500);
            schedule_local_notif("You got more free episodes!", remaining_seconds);
        }
    }
    $('#counters').show();
};

format_timer = function(seconds){
    // returns HH:MM:SS format
    var hours = Math.floor(seconds / 3600);
    seconds -= hours * 3600;
    var minutes = Math.floor(seconds / 60);
    seconds -= minutes * 60;
    if (hours < 10) hours = '0' + hours;
    if (minutes < 10) minutes = '0' + minutes;
    if (seconds < 10) seconds = '0' + seconds;
    return hours + ':' + minutes + ':' + seconds;
};

format_readable_timer = function(seconds){
    // 1 hour, or 3 minutes, or 25 seconds
    var n = seconds;
    var units = 'seconds';
    if (seconds > 5400) {
        n = Math.round(seconds / 3600);
        units = 'hours';
    } else if (seconds > 90){
        n = Math.round(seconds / 60);
        units = 'minutes';
    }

    $('.remaining_time').html(n);
    $('.time_units').html(units);
};


// -------------------------------
// PART SIX: define click handlers for elements
// -------------------------------

BUTTONS = {};

BUTTONS.start_first_episode = function(){
    next();
}
BUTTONS.start = function(){
    if (IS_HOURLY && READY_TO_PLAY){
        next();
    } else if (IS_HOURLY === false && GAMESTATE.FREE_EPISODES > 0) {
        LOG('using free without confirmation...');
        GAMESTATE.FREE_EPISODES -= 1;
        sync_state_then_callback(next);
    } else if (GAMESTATE.PASSES > 0) {
        LOG('use paid?');
        SHOW_MODAL('modal_confirm_next');
    } else {
        LOG('buy passes?');
        SHOW_MODAL('modal_buy_passes');
    }
}

BUTTONS.cancel_modal_buy_passes = function(){
    HIDE_MODAL('modal_buy_passes');
}
BUTTONS.like = function(){
    LOG('tapped the like button!');
}

BUTTONS.share = function(){
    LOG('Yeah Mr. White! SHARING');
}

BUTTONS.alerts = function(){
    LOG('WAKEY WAKY');
}

BUTTONS.email = function(){
    // we need an in-app alert...
    // wire up the actual modal functionality and show the dialog
    SHOW_MODAL('ask_for_email');
}

BUTTONS.email_ok = function(){
    HIDE_MODAL('ask_for_email');
    LOG($('#user_email').val());
}

BUTTONS.show_console = function(){
    SHOW_MODAL('modal_console');
}

BUTTONS.dismiss_console = function(){
    HIDE_MODAL('modal_console');
}

BUTTONS.menu = function(){
    // quit to the catalog
    LOG('menu!');
    window.location.href = 'episode:///';
}
BUTTONS.cancel_next = function(){
    HIDE_MODAL('modal_confirm_next');
}

BUTTONS.go_to_pass_shop = function(){
    show_native_store();
//    var query = window.location.search;
//    window.location.href = 'shop_bundle.html' + query;
}

BUTTONS.go_to_pass_shop2 = function(){
    show_native_store();
//    var query = window.location.search;
//    window.location.href = 'shop_bundle.html' + query;
}

BUTTONS.play_another_story = function(){
    window.location.href = 'episode:///';
}

BUTTONS.close_button = function(){
    B('close');
}

BUTTONS.close_button_2 = function(){
    B('close');
}

BUTTONS.spend_pass = function(){
    LOG('use pass.');
    GAMESTATE.PASSES -= 1;

    var funnelStep = Q('{"f":"spendPass", "s":"' + GAMESTATE.StoryID + '/' + GAMESTATE.ChapterNum + '"}');
    var extraQuery = '&funnelstep=' + funnelStep;

    sync_state_then_callback(next, extraQuery);
}

buy_passes_iap = function(iap_identifier) {
    var iap = IAPS[iap_identifier];
    if (!iap){
        REMOTE_LOG("invalid iap: " + iap_identifier);
        return;
    }

    SYNC_DISABLED = true;

    var js_state = Q('{"countersIncrement":{"PASSES":' + iap.passes
                     + '}}');
    B('iap?identifier=' + iap_identifier + '&set=' + js_state);
}

BUTTONS.buy_tier_1 = function(){
    buy_passes_iap('iap_1_premiumchoice_1');
}

BUTTONS.buy_tier_2 = function(){
    buy_passes_iap('iap_1_premiumchoice_2');
}

BUTTONS.buy_tier_5 = function(){
    buy_passes_iap('iap_1_premiumchoice_5');
}

BUTTONS.buy_tier_10 = function(){
    buy_passes_iap('iap_1_premiumchoice_10');
}

BUTTONS.back = function(){
    // from the shop to the gate
    var query = window.location.search;
    window.location.href = "gate_bundle.html" + query;
}

BUTTONS.playthroughs = function(){
    B('playthroughs');
}

BUTTONS.contact = function(){
    var URL = 'http://pg-story.appspot.com/ask/' + GAMESTATE.StoryID + '/';
    URL += '?pgid=' + GAMESTATE.PGID;
    window.location.href = URL;
}

BUTTONS.faq = function(){
    var URL = 'http://pg-story.appspot.com/messages/' + GAMESTATE.StoryID + '/';
    URL += '?pgid=' + GAMESTATE.PGID;
    window.location.href = URL;
}

// Referral

BUTTONS.applyreferral = function(){
    SYNC_DISABLED = true;

    // get value from text field
    referral_code = $('#referral_input').val();
    query_referral_code = Q(referral_code)
    B('applyreferral?referralCode=' + query_referral_code);
}

did_applyreferral = function(result){
    if (result.PASSES != undefined) {
        GAMESTATE.PASSES = result.PASSES;
    }
    SYNC_DISABLED = false;

    if ('failure' === result.referralResult) {
        // iOS code already presents the error to the user; so, do nothing
    } else if ('success' === result.referralResult) {
        // switch to the version of this view which does not have the text box
        sync_state_then_callback(did_sync_referral_progress)
    } {
        // ???
    }
}

did_sync_referral_progress = function() {
    B('referral');
}

BUTTONS.referral = function(){
    B('referral');
}

BUTTONS.referral_popup = function(){
    B('referral?popup=true');
}

BUTTONS.sharereferral = function(){
    // Legally cannot provide a default message
    messageQuery = Q(' ');
    B('sharereferral?message=' + messageQuery);
}

did_sharereferral = function(){}

// Facebook
BUTTONS.login_fb = function(){
    B('fblogin');
}

did_fb_login = function(){}

BUTTONS.reloader = function() {
    location.reload(true);
};

did_reloader = function() {};

// Native Store
show_native_store = function() {
    // Show the store
    // If an iap completes, do_render to make sure we absorb any changes instead of overwriting them
    B('store/do_render');
}

// -------------------------------
// PART SEVEN: Parameters
// -------------------------------

FREE_EPISODES_PER_DAY = 2;
STORYDATA = null; // fetched via JSONP
GAMESTATE = null;
SYNC_DISABLED = false;
